#!/usr/local/bin/wish -f
#
# Copyright (c) 1995 The XD Group, Free University of Berlin. All rights reserved.
#
# $Header: /scratch/fock/cvsroot/xd/xdgraph/tcl/gui.tcl,v 2.6 1999/05/13 10:32:39 richter Rel $
#
# $Log: gui.tcl,v $
# Revision 2.6  1999/05/13 10:32:39  richter
# do not check the Tk version here,
# the file selector is choosen by $xd_use_fsbox
#
# Revision 2.5  1999/03/23 21:31:00  richter
# use native file selector for Tk versions 8 upwards
#
# Revision 2.4  1997/04/02  13:45:18  richter
# source VMS specific help menu file if necessary
#
# Revision 2.3  1996/11/07  17:09:40  richter
# set a default focus for Tk 3.*
#
# Revision 2.2  1996/09/14  09:34:47  richter
# Control-U binding for entry widgets
# open the correct file in write_logfile
#
# Revision 2.1  1996/09/06  12:42:16  richter
# new proc write_logfile
#
# Revision 2.0  1995/05/12  11:54:38  richter
# First semi-public beta release
#
#

source $xd_library/CSBox_3.tcl

if {$xd_use_fsbox == "tcl"} {
    proc FSBox {w args} {
	set types {
	    {{XD Grid Files}     {.grd} }
	    {{XD Bondpath Files} {.pth} }
	    {{All Files}         *      }
	}

	set cmd [list tk_getOpenFile -filetypes $types]
	if {$w != ""} {
	    lappend cmd -parent $w
	}
	return [eval $cmd]
    }
} else {
    source $xd_library/FSBox_3.tcl
}
source $xd_library/guiContainer.tcl
source $xd_library/guiConfigure.tcl

source $xd_library/guiFileMenu.tcl
##source $xd_library/guiAssistMenu.tcl
source $xd_library/guiCreateMenu.tcl
source $xd_library/guiViewsMenu.tcl
source $xd_library/guiObjectsMenu.tcl
if {$xd_unix} {
    source $xd_library/guiHelpMenu.tcl
} else {
    source $xd_library/guiHelpMenu_VMS.tcl
}

proc mkGUI {} {
    global xd_tk_major in out next_event

    wm title . "xdGraph Control"

    if {$xd_tk_major >= 4} {
	#    option add *highlightThickness 0 startupFile
    } else {
	#    option add *Label.borderwidth 2 startupFile
	option add *Entry.relief sunken startupFile
	option add *Label.relief flat startupFile
	option add *Listbox.relief sunken startupFile
    }

    option add *Entry.background \#b98e8e startupFile
    option add *Listbox.background \#b98e8e startupFile
    option add *Button.padX 2 startupFile

    bind Entry <Left>  {%W icursor [expr [%W index insert] - 1]}
    bind Entry <Right> {%W icursor [expr [%W index insert] + 1]}
    bind Entry <Control-a> {%W icursor 0}
    bind Entry <Control-e> {%W icursor end}
    bind Entry <Control-u> {%W delete 0 end}
	
    set in ""

#
# outermost frame of the control window
#
    frame .control
#
#####################################
# frame for output (text + scrollbar)
#
    frame .control.out
#
#
# text output
#
    set out [text .control.out.text -height 24 -width 80  -wrap word -bd 2 \
		 -relief sunken -yscrollcommand ".control.out.scroll set"]
    $out tag config cmd  -foreground blue
    $out tag config err  -foreground red
    $out tag config info -foreground black
#
#
# scrollbar for the text window
#
    scrollbar .control.out.scroll -command ".control.out.text yview"
#
#################################
# frame for input (label + entry)
#
    frame .control.in -bd 2 -relief flat
#
#
# label (prompt)
#
    label .control.in.label -text "Command: "
#
#
# entry
#
    entry .control.in.entry -textvariable in
    bind .control.in.entry <Return> {catch doCommand}
    bind .control.in.entry <Up> {recall %W -1}
    bind .control.in.entry <Down> {recall %W +1}
    set next_event 0

#
# abort
#
    button .control.in.abort -text Abort -command "set abort 1"
#
#################################
# frame for the menu
#
    set mb [frame .control.mbar -relief raised -bd 2]

    guiFileMenu $mb.file
##    guiAssistMenu $mb.assist
    guiCreateMenu $mb.create
    guiViewsMenu $mb.views
    guiObjectsMenu $mb.objects
    guiHelpMenu $mb.help
    if {$xd_tk_major < 4} {
##	tk_menuBar $mb $mb.file $mb.assist $mb.create $mb.views $mb.objects $mb.help
	tk_menuBar $mb $mb.file $mb.create $mb.views $mb.objects $mb.help
    }
#
#
################
#
# pack the stuff
#
    pack .control.out.text .control.out.scroll -side left -expand yes -fill y
#
    pack .control.in.label -side left
    pack .control.in.entry -side left -expand yes -fill both
    pack .control.in.abort -side right
#
    pack $mb.file -side left
##    pack $mb.assist -side left
    pack $mb.create -side left
    pack $mb.views -side left
    pack $mb.objects -side left
    pack $mb.help -side right
#
    pack $mb -side top -fill x
    pack .control.in -side bottom -fill x
    pack .control.out -side top -expand yes -fill both
    pack .control -side top -expand yes -fill both
#
#
# set the focus to the entry widget
#
    if {$xd_tk_major < 4} {
	focus default .control.in.entry
    }
    focus .control.in.entry
}

proc logMessage {tag text} {
    global out

    set pos [$out index end-1chars]
    $out insert end $text
    if {$tag != "none" && $tag != "inf"} {
	$out tag add $tag $pos end-1chars
    }
    $out yview -pickplace end
}

proc write_logfile {{file xdgraph.log}} {
    global out

    set L [open $file w]
    set end [expr int([$out index end])]
    for {set i 1} {$i <= $end} {incr i} {
	set line "$i.0"
	set text [$out get $line "$line lineend"]
	set tag [$out tag names $line]
	switch -exact -- $tag {
	    cmd {}
	    err {set text "#error: $text"}
	    default {set text "# $text"}
	}
	puts $L $text
    }
    close $L
}
    
proc object_buttons {root arguments} {
    set w [frame $root.buttons -bd 5]

    foreach data $arguments {
	set lab [lindex $data 0]
	set cmd [lindex $data 1]
	set options [lindex $data 2]
	set f [frame $w.[string tolower $lab] -bd 1 -relief flat]
	set b [button $f.[string tolower $lab] -text $lab -width 6 \
		   -command "$cmd $root"]
	pack $b -side left -expand yes
	if {$options == "default"} {
	    $f configure -relief sunken
	    bind $root <Return> "$cmd $root"
	}
	pack $f -side left -expand yes -ipadx 2 -ipady 2
    }
}

proc recall {w incr} {
    global next_event

    set old $next_event
    incr next_event $incr
    if {[catch {history event $next_event} result] == 0} {
	$w delete 0 end
	$w insert end $result
    } else {
	set next_event $old
    }
}
