#!/usr/local/bin/wish -f
#
# Copyright (c) 1995 The XD Group, Free University of Berlin. All rights reserved.
#
# $Header: /scratch/fock/cvsroot/xd/xdgraph/tcl/guiContainer.tcl,v 2.4 1996/11/13 13:19:56 richter Rel $
#
# $Log: guiContainer.tcl,v $
# Revision 2.4  1996/11/13 13:19:56  richter
# use a self-made toggle button if on can choose between on and off only
#
# Revision 2.3  1996/09/14  09:36:32  richter
# new option -required to be able to mark required fields (currently by BG)
# moved Select_file Select_color and trace_uservalue to guiConfigure
#
# Revision 2.2  1996/09/06  14:50:59  richter
# accept an external command to construct default values
#
# Revision 2.1  1996/09/06  12:44:23  richter
# lots of changes
#
# Revision 2.0  1995/05/12  11:54:38  richter
# First semi-public beta release
#
#


#
# container PATHNAME
#    -label {LABEL {SUBLABEL ...}}
#    -var {{DATAKEY ...} BASENAME}
#    ?-entry ENTRY?
#    ?-help HELPKEY?
#    ?-onoff ONOFFKEY?
#    ?-choose {{NAME VALUE} ...}?
#    ?-selectCmd SELECTCMD?
#    ?-required BOOL?
#
proc container {args} {
    global xd_tk_major
#
    set w [lindex $args 0]
    set args [lreplace $args 0 0]
    set label LABEL
    set onoff ""
    set choose ""
    set selectCmd ""
    set help ""
    set entry 1
    set defArray ""
    set cfgArray ""
    set key ""
    set readonly 0
    set width 20
    set matrix 0
    set defCmd ""
    set required 0
    while {[llength $args] >= 2} {
	set option [lindex $args 0]
	set value  [lindex $args 1]
	set args [lreplace $args 0 1]
	switch -glob -- $option {
	    -label {
		set label [lindex $value 0]
		set sublablist [lindex $value 1]
	    }
	    -var {
		set keyList [lindex $value 0]
		if {[llength $keyList] > 2} {
		    set width 10
		    if {[llength $keyList] > 4} {
			set matrix 1
		    }
		}
		set baseName [lindex $value 1]
		foreach key $keyList {
		    set cfgArray cfg${baseName}
		    set curArray cur${baseName}
		    set defArray def${baseName}
		    set cfgName ${cfgArray}($key)
		    set curName ${curArray}($key)
		    set defName ${defArray}($key)
		    upvar #0 $curName cur $cfgName cfg $defName def
		}
	    }
	    -entry {
		set entry $value
	    }
	    -help {
		set help $value
	    }
	    -onoff {
		set onoff $value
	    }
	    -choose {
		set choose $value
	    }
	    -select* {
		set selectCmd $value
	    }
	    -readonly {
		set readonly $value
	    }
	    -required {
		set required $value
	    }
	    -defCmd {
		set defCmd $value
	    }
	}
    }
#
    frame $w -bd 2 -relief groove
    frame $w.top
    frame $w.top.left -width 48 -bd 0
    label $w.top.label -text $label -anchor sw -cursor question_arrow
    if {$onoff != "" && !$readonly} {
	set wList [list $w.mid.entry $w.mid.default $w.mid.undo $w.mid.select $w.mid.choose]
	set onoffcmd [list container_onoff ${cfgArray}($onoff) $wList]
	button $w.top.onoff -bitmap xd_onoff -relief groove \
	    -command $onoffcmd
	pack [frame $w.top.left.filler -width 24] -side left \
	    -expand yes -fill both
	pack $w.top.onoff -in $w.top.left -side right
    } else {
	set onoff ""
    }
    pack $w.top.left -side left -anchor se
    pack $w.top.label -side left -expand yes -fill x -anchor sw
    if {$help != ""} {
        bind $w.top.label <ButtonRelease-1> "xdhelp $help"
    }

    frame $w.mid
    frame $w.container
    frame $w.mid.left
    if {$entry} {
	set femaster [frame $w.mid.entry]
	set fe $femaster
	set i -1
	foreach key $keyList {
	    set cfgName ${cfgArray}($key)
	    set sublab [lindex $sublablist [incr i]]
	    if {$matrix && (($i % 3) == 0)} {
		set fe [frame $femaster.frame$i]
		pack $fe -side top -expand yes -fill both
	    }
	    if {$sublab != ""} {
		if {$i > 0} {
		    label $fe.sublab$i -text "  $sublab:"
		} else {
		    label $fe.sublab$i -text "$sublab:"
		}
		pack $fe.sublab$i -side left
	    }
	    if {!$readonly} {
		if {$required} {
		    entry $fe.$key -textvariable $cfgName -width $width -bg \#b9b98e 
		} else {
		    entry $fe.$key -textvariable $cfgName -width $width -bg \#b98e8e
		}
	    } else {
		label $fe.$key -textvariable $cfgName  -width $width -anchor w
	    }
	    pack $fe.$key -side left -expand yes -fill x -anchor w
	    lappend wList $fe.$key
	}
    }
    if {$defCmd != "" && !$readonly} {
	button $w.mid.default -bitmap xd_default -relief groove \
	    -command [list do_extern_defcmd $defCmd $keyList $cfgArray $defArray]
	if {$xd_tk_major >= 4} {
	    $w.mid.default configure -takefocus 0
	}
    } elseif {$defArray != "" && !$readonly} {
	button $w.mid.default -bitmap xd_default -relief groove \
	    -command [list container_reset $keyList $cfgArray $defArray]
	if {$xd_tk_major >= 4} {
	    $w.mid.default configure -takefocus 0
	}
    } else {
	frame $w.mid.default -width 24
    }
    if {$cfgArray != "" && !$readonly} {
	button $w.mid.undo -bitmap xd_undo -relief groove \
	    -command [list container_reset $keyList $cfgArray $curArray]
	if {$xd_tk_major >= 4} {
	    $w.mid.undo configure -takefocus 0
	}
    } else {
	frame $w.mid.undo -width 24
    }
    pack $w.mid.undo $w.mid.default -in $w.mid.left -side right
    pack $w.mid.left -side left
    if {$entry} {
	pack $w.mid.entry -side left -expand yes -fill x -anchor w
    } else {
	pack $w.container -in $w.mid -side left -anchor nw
    }
    if {$choose != "" && !$readonly} {
	set mbut $w.mid.choose
	if {$choose == {{on on} {off off}}} {
	    button $mbut -bitmap xd_onoff -relief groove \
		-command [list container_toggle $cfgName]
	} else {
	    set m $mbut.m
	    if {$xd_tk_major >= 4} {
		menubutton $mbut -bitmap xd_choose -menu $m -relief groove\
		    -highlightthickness 2
		menu $m -tearoff 0
	    } else {
		menubutton $mbut -bitmap xd_choose -menu $m -relief groove
		menu $m
	    }
	    foreach i $choose {
		set lab [lindex $i 0]
		set val [lindex $i 1]
		$m add command -label $lab -command [list set $cfgName $val]
	    }
	}
	pack $mbut -side left
    }
    if {$selectCmd != "" && !$readonly} {
	button $w.mid.select -bitmap xd_select -relief groove \
	    -command [list $selectCmd $w $cfgArray $keyList]
	pack $w.mid.select -side left
    }

    pack $w.top $w.mid -side top -anchor w  -expand yes -fill both
    if {$entry} {
	frame $w.bot_left -width 36p
	pack $w.bot_left -side left
	pack $w.container -side left -anchor nw -expand yes -fill both
    }
    if {$onoff != ""} {
	set onoffcmd [list container_onoff ${cfgArray}($onoff) $wList]
	$w.top.onoff configure -command $onoffcmd
	eval $onoffcmd
	eval $onoffcmd
    }
}

proc container_toggle {varName} {
    upvar #0 $varName var

    if {$var} {
	set var off
    } else {
	set var on
    }
}
proc container_onoff {varName wList} {
    upvar #0 $varName var

    set var [expr ! $var]
    if {$var} {
	set cmd container_enable
    } else {
	set cmd container_disable
    }
    foreach w $wList {
	$cmd $w
    }
}

proc container_enable {w} {
    if {[winfo exists $w]} {
	if {[winfo class $w] == "Button"} {
	    $w config -state normal
	} elseif {[winfo class $w] == "Entry"} {
	    $w config -state normal -bg \#b98e8e
	}
    }
}

proc container_disable {w} {
    if {[winfo exists $w]} {
	if {[winfo class $w] == "Button"} {
	    $w config -state disabled
	} elseif {[winfo class $w] == "Entry"} {
	    $w config -state disabled -bg \#b0b0b0
	}
    }
}

proc container_reset {keyList cfgArray defArray} {
    foreach key $keyList {
	set cfgName ${cfgArray}
	set defName ${defArray}
	upvar #0 $cfgName cfg $defName def
	set cfg($key) $def($key)
    }
}

proc do_extern_defcmd {defCmd keyList cfgArray defArray} {
    foreach key $keyList {
	set cfgName ${cfgArray}
	set defName ${defArray}
	upvar #0 $cfgName cfg $defName def
	set cfg($key) [eval $defCmd $key $def($key)]
    }
}


proc complete_objectname {name} {
    global active

    switch -glob $name {
	""   {
	    return ""
	}
	:*:* {
	    return $name
	}
	:* {
	    if {$active(dataset) != ""} {
		return "$active(dataset)$name"
	    }
	}
	default {
	    if {$active(dataset) != ""} {
		return "$active(dataset):$name"
	    }
	}
    }
    error "no active dataset"
}

